@@ -8,134 +8,9 @@ from twisted.python.failure import Failure
 from twisted.internet import reactor, defer, error
 
 from buildbot import interfaces, locks
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
-from buildbot.status.builder import Results, BuildRequestStatus
+from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION, RETRY
+from buildbot.status.builder import Results
 from buildbot.status.progress import BuildProgress
-from buildbot.process.properties import Properties
-
-class BuildRequest:
-    """I represent a request to a specific Builder to run a single build.
-
-    I have a SourceStamp which specifies what sources I will build. This may
-    specify a specific revision of the source tree (so source.branch,
-    source.revision, and source.patch are used). The .patch attribute is
-    either None or a tuple of (patchlevel, diff), consisting of a number to
-    use in 'patch -pN', and a unified-format context diff.
-
-    Alternatively, the SourceStamp may specify a set of Changes to be built,
-    contained in source.changes. In this case, I may be mergeable with other
-    BuildRequests on the same branch.
-
-    I may be part of a BuildSet, in which case I will report status results
-    to it.
-
-    I am paired with a BuildRequestStatus object, to which I feed status
-    information.
-
-    @type source: a L{buildbot.sourcestamp.SourceStamp} instance.   
-    @ivar source: the source code that this BuildRequest use
-
-    @type reason: string
-    @ivar reason: the reason this Build is being requested. Schedulers
-                  provide this, but for forced builds the user requesting the
-                  build will provide a string.
-
-    @type properties: Properties object
-    @ivar properties: properties that should be applied to this build
-
-    @ivar status: the IBuildStatus object which tracks our status
-
-    @ivar submittedAt: a timestamp (seconds since epoch) when this request
-                       was submitted to the Builder. This is used by the CVS
-                       step to compute a checkout timestamp.
-    """
-
-    source = None
-    builder = None
-    startCount = 0 # how many times we have tried to start this build
-
-    implements(interfaces.IBuildRequestControl)
-
-    def __init__(self, reason, source, builderName=None, properties=None):
-        # TODO: remove the =None on builderName, it is there so I don't have
-        # to change a lot of tests that create BuildRequest objects
-        assert interfaces.ISourceStamp(source, None)
-        self.reason = reason
-        self.source = source
-
-        self.properties = Properties()
-        if properties:
-            self.properties.updateFromProperties(properties)
-
-        self.start_watchers = []
-        self.finish_watchers = []
-        self.status = BuildRequestStatus(source, builderName)
-
-    def canBeMergedWith(self, other):
-        return self.source.canBeMergedWith(other.source)
-
-    def mergeWith(self, others):
-        return self.source.mergeWith([o.source for o in others])
-
-    def mergeReasons(self, others):
-        """Return a reason for the merged build request."""
-        reasons = []
-        for req in [self] + others:
-            if req.reason and req.reason not in reasons:
-                reasons.append(req.reason)
-        return ", ".join(reasons)
-
-    def waitUntilFinished(self):
-        """Get a Deferred that will fire (with a
-        L{buildbot.interfaces.IBuildStatus} instance when the build
-        finishes."""
-        d = defer.Deferred()
-        self.finish_watchers.append(d)
-        return d
-
-    # these are called by the Builder
-
-    def requestSubmitted(self, builder):
-        # the request has been placed on the queue
-        self.builder = builder
-
-    def buildStarted(self, build, buildstatus):
-        """This is called by the Builder when a Build has been started in the
-        hopes of satifying this BuildRequest. It may be called multiple
-        times, since interrupted builds and lost buildslaves may force
-        multiple Builds to be run until the fate of the BuildRequest is known
-        for certain."""
-        for o in self.start_watchers[:]:
-            # these observers get the IBuildControl
-            o(build)
-        # while these get the IBuildStatus
-        self.status.buildStarted(buildstatus)
-
-    def finished(self, buildstatus):
-        """This is called by the Builder when the BuildRequest has been
-        retired. This happens when its Build has either succeeded (yay!) or
-        failed (boo!). TODO: If it is halted due to an exception (oops!), or
-        some other retryable error, C{finished} will not be called yet."""
-
-        for w in self.finish_watchers:
-            w.callback(buildstatus)
-        self.finish_watchers = []
-
-    # IBuildRequestControl
-
-    def subscribe(self, observer):
-        self.start_watchers.append(observer)
-    def unsubscribe(self, observer):
-        self.start_watchers.remove(observer)
-
-    def cancel(self):
-        """Cancel this request. This can only be successful if the Build has
-        not yet been started.
-
-        @return: a boolean indicating if the cancel was successful."""
-        if self.builder:
-            return self.builder.cancelBuildRequest(self)
-        return False
 
 
 class Build:
@@ -168,6 +43,7 @@ class Build:
     reason = "changes"
     finished = False
     results = None
+    stopped = False
 
     def __init__(self, requests):
         self.requests = requests
@@ -182,6 +58,8 @@ class Build:
         self.currentStep = None
         self.slaveEnvironment = {}
 
+        self.terminate = False
+
     def setBuilder(self, builder):
         """
         Set the given builder as our builder.
@@ -193,6 +71,9 @@ class Build:
     def setLocks(self, locks):
         self.locks = locks
 
+    def setSlaveEnvironment(self, env):
+        self.slaveEnvironment = env
+
     def getSourceStamp(self):
         return self.source
 
@@ -214,23 +95,14 @@ class Build:
     def allFiles(self):
         # return a list of all source files that were changed
         files = []
-        havedirs = 0
         for c in self.allChanges():
             for f in c.files:
                 files.append(f)
-            if c.isdir:
-                havedirs = 1
         return files
 
     def __repr__(self):
         return "<Build %s>" % (self.builder.name,)
 
-    def __getstate__(self):
-        d = self.__dict__.copy()
-        if d.has_key('remote'):
-            del d['remote']
-        return d
-
     def blamelist(self):
         blamelist = []
         for c in self.allChanges():
@@ -276,12 +148,18 @@ class Build:
         for rq in self.requests:
             props.updateFromProperties(rq.properties)
 
+        # and finally, from the SourceStamp, which has properties via Change
+        for change in self.source.changes:
+            props.updateFromProperties(change.properties)
+
         # now set some properties of our own, corresponding to the
         # build itself
-        props.setProperty("buildername", self.builder.name, "Build")
         props.setProperty("buildnumber", self.build_status.number, "Build")
         props.setProperty("branch", self.source.branch, "Build")
         props.setProperty("revision", self.source.revision, "Build")
+        props.setProperty("repository", self.source.repository, "Build")
+        props.setProperty("project", self.source.project, "Build")
+        self.builder.setupProperties(props)
 
     def setupSlaveBuilder(self, slavebuilder):
         self.slavebuilder = slavebuilder
@@ -336,15 +214,15 @@ class Build:
             self.setupBuild(expectations) # create .steps
         except:
             # the build hasn't started yet, so log the exception as a point
-            # event instead of flunking the build. TODO: associate this
-            # failure with the build instead. this involves doing
+            # event instead of flunking the build. 
+            # TODO: associate this failure with the build instead. 
+            # this involves doing
             # self.build_status.buildStarted() from within the exception
             # handler
             log.msg("Build.setupBuild failed")
             log.err(Failure())
             self.builder.builder_status.addPointEvent(["setupBuild",
-                                                       "exception"],
-                                                      color="purple")
+                                                       "exception"])
             self.finished = True
             self.results = FAILURE
             self.deferred = None
@@ -378,7 +256,7 @@ class Build:
         # add a count to the loser until it is unique.
         self.steps = []
         self.stepStatuses = {}
-        stepnames = []
+        stepnames = {}
         sps = []
 
         for factory, args in self.stepFactories:
@@ -391,16 +269,19 @@ class Build:
                 raise
             step.setBuild(self)
             step.setBuildSlave(self.slavebuilder.slave)
-            step.setDefaultWorkdir(self.workdir)
+            if callable (self.workdir):
+                step.setDefaultWorkdir (self.workdir (self.source))
+            else:
+                step.setDefaultWorkdir (self.workdir)
             name = step.name
-            count = 1
-            while name in stepnames and count < 100:
+            if stepnames.has_key(name):
+                count = stepnames[name]
                 count += 1
+                stepnames[name] = count
                 name = step.name + "_%d" % count
-            if name in stepnames:
-                raise RuntimeError("duplicate step '%s'" % step.name)
+            else:
+                stepnames[name] = 0
             step.name = name
-            stepnames.append(name)
             self.steps.append(step)
 
             # tell the BuildStatus about the step. This will create a
@@ -435,6 +316,11 @@ class Build:
         self.build_status.setBlamelist(self.blamelist())
         self.build_status.setProgress(self.progress)
 
+        # gather owners from build requests
+        owners = [r.properties['owner'] for r in self.requests
+                  if r.properties.has_key('owner')]
+        if owners: self.setProperty('owners', owners, self.reason)
+
         self.results = [] # list of FAILURE, SUCCESS, WARNINGS, SKIPPED
         self.result = SUCCESS # overall result, may downgrade after each step
         self.text = [] # list of text string lists (text2)
@@ -443,9 +329,19 @@ class Build:
         """This method is called to obtain the next BuildStep for this build.
         When it returns None (or raises a StopIteration exception), the build
         is complete."""
+        if self.stopped:
+            return None
         if not self.steps:
             return None
-        return self.steps.pop(0)
+        if self.terminate:
+            while True:
+                s = self.steps.pop(0)
+                if s.alwaysRun:
+                    return s
+                if not self.steps:
+                    return None
+        else:
+            return self.steps.pop(0)
 
     def startNextStep(self):
         try:
@@ -465,8 +361,8 @@ class Build:
             return # build was interrupted, don't keep building
         terminate = self.stepDone(results, step) # interpret/merge results
         if terminate:
-            return self.allStepsDone()
-        self.startNextStep()
+            self.terminate = True
+        return self.startNextStep()
 
     def stepDone(self, result, step):
         """This method is called when the BuildStep completes. It is passed a
@@ -491,7 +387,6 @@ class Build:
             if step.flunkOnFailure:
                 self.result = FAILURE
             if step.haltOnFailure:
-                self.result = FAILURE
                 terminate = True
         elif result == WARNINGS:
             if step.warnOnWarnings:
@@ -502,6 +397,9 @@ class Build:
         elif result == EXCEPTION:
             self.result = EXCEPTION
             terminate = True
+        elif result == RETRY:
+            self.result = RETRY
+            terminate = True
         return terminate
 
     def lostRemote(self, remote=None):
@@ -528,44 +426,33 @@ class Build:
             return
         # TODO: include 'reason' in this point event
         self.builder.builder_status.addPointEvent(['interrupt'])
+        self.stopped = True
         self.currentStep.interrupt(reason)
-        if 0:
-            # TODO: maybe let its deferred do buildFinished
-            if self.currentStep and self.currentStep.progress:
-                # XXX: really .fail or something
-                self.currentStep.progress.finish()
-            text = ["stopped", reason]
-            self.buildFinished(text, "red", FAILURE)
 
     def allStepsDone(self):
         if self.result == FAILURE:
-            color = "red"
             text = ["failed"]
         elif self.result == WARNINGS:
-            color = "orange"
             text = ["warnings"]
         elif self.result == EXCEPTION:
-            color = "purple"
             text = ["exception"]
         else:
-            color = "green"
             text = ["build", "successful"]
         text.extend(self.text)
-        return self.buildFinished(text, color, self.result)
+        return self.buildFinished(text, self.result)
 
     def buildException(self, why):
         log.msg("%s.buildException" % self)
         log.err(why)
-        self.buildFinished(["build", "exception"], "purple", FAILURE)
+        self.buildFinished(["build", "exception"], FAILURE)
 
-    def buildFinished(self, text, color, results):
+    def buildFinished(self, text, results):
         """This method must be called when the last Step has completed. It
         marks the Build as complete and returns the Builder to the 'idle'
         state.
 
-        It takes three arguments which describe the overall build status:
-        text, color, results. 'results' is one of SUCCESS, WARNINGS, or
-        FAILURE.
+        It takes two arguments which describe the overall build status:
+        text, results. 'results' is one of SUCCESS, WARNINGS, or FAILURE.
 
         If 'results' is SUCCESS or WARNINGS, we will permit any dependant
         builds to start. If it is 'FAILURE', those builds will be
@@ -578,10 +465,9 @@ class Build:
 
         log.msg(" %s: build finished" % self)
         self.build_status.setText(text)
-        self.build_status.setColor(color)
         self.build_status.setResults(results)
         self.build_status.buildFinished()
-        if self.progress:
+        if self.progress and results == SUCCESS:
             # XXX: also test a 'timing consistent' flag?
             log.msg(" setting expectations for next time")
             self.builder.setExpectations(self.progress)
